@@ -1,13 +1,15 @@
 import logging
 import tempfile
-from typing import Optional, Union, Any, Dict, List
+from typing import Any, Dict, List, Optional, Union
 
+import numpy
 import pandas
 from sklearn.base import BaseEstimator, RegressorMixin
 
-from hts import model as hts_models, defaults
-from hts._t import Transform, NodesT, ExogT, Model
-from hts.core.exceptions import MissingRegressorException, InvalidArgumentException
+from hts import defaults
+from hts import model as hts_models
+from hts._t import ExogT, MethodT, ModelT, NodesT, TimeSeriesModelT, Transform
+from hts.core.exceptions import InvalidArgumentException, MissingRegressorException
 from hts.core.result import HTSResult
 from hts.core.utils import _do_fit, _do_predict, _model_mapping_to_iterable
 from hts.functions import to_sum_mat
@@ -43,7 +45,7 @@ class HTSRegressor(BaseEstimator, RegressorMixin):
         The dataframe containing the nodes and edges specified above
 
     revision_method : str
-        One of the revisions methods specified in ...
+        One of: ``"OLS", "WLSS", "WLSV", "FP", "PHA", "AHP", "BU", "NONE"``
 
     models : dict
         Dictionary that holds the trained models
@@ -60,31 +62,24 @@ class HTSRegressor(BaseEstimator, RegressorMixin):
     model_instance : TimeSeriesModel
         Reference to the class implementing the actual time series model
 
-    Methods
-    -------
-    fit(self, df: pandas.DataFrame, nodes: Dict, exogenous: List = None, root: str ='total', **fit_args)
-        Fits underlying models to the data
-
-    predict(self, exogenous: pandas.DataFrame = None, steps_ahead: int = 10)
-        Predicts the n-step ahead forecast with exogenous variables. Exogenous variables are required if models were
-        fit using them
     """
 
-    def __init__(self,
-                 model: str = defaults.MODEL,
-                 revision_method: str = defaults.REVISION,
-                 transform: Optional[Union[Transform, bool]] = None,
-                 n_jobs: int = defaults.N_PROCESSES,
-                 low_memory: bool = defaults.LOW_MEMORY,
-                 **kwargs: Any
-                 ):
+    def __init__(
+        self,
+        model: str = defaults.MODEL,
+        revision_method: str = defaults.REVISION,
+        transform: Optional[Union[Transform, bool]] = None,
+        n_jobs: int = defaults.N_PROCESSES,
+        low_memory: bool = defaults.LOW_MEMORY,
+        **kwargs: Any,
+    ):
         """
         Parameters
         ----------
         model : str
             One of the models supported by ``hts``. These can be found
         revision_method : str
-            The revision method to be used
+            The revision method to be used. One of: ``"OLS", "WLSS", "WLSV", "FP", "PHA", "AHP", "BU", "NONE"``
         transform : Boolean or NamedTuple
             If True, ``scipy.stats.boxcox`` and ``scipy.special._ufuncs.inv_boxcox`` will be applied prior and after
             fitting.
@@ -100,61 +95,73 @@ class HTSRegressor(BaseEstimator, RegressorMixin):
         """
 
         self.model = model
-        self.method = revision_method
-        self.n_jobs = n_jobs
-        self.low_memory = low_memory
+        self.method: str = revision_method
+        self.n_jobs: int = n_jobs
+        self.low_memory: bool = low_memory
         if self.low_memory:
-            self.tmp_dir = tempfile.mkdtemp(prefix='hts_')
+            self.tmp_dir: Optional[str] = tempfile.mkdtemp(prefix="hts_")
         else:
             self.tmp_dir = None
         self.transform = transform
 
-        self.sum_mat = None
-        self.nodes = None
-        self.model_instance = None
-        self.exogenous = False
-        self.revision_method = None
-        self.hts_result = HTSResult()
+        self.sum_mat: Optional[numpy.ndarray] = None
+        self.nodes: Optional[NodesT] = None
+        self.model_instance: Optional[TimeSeriesModelT] = None
+        self.exogenous: bool = False
+        self.revision_method: Optional[RevisionMethod] = None
+        self.hts_result: HTSResult = HTSResult()
         self.model_args = kwargs
 
-    def __init_hts(self,
-                   nodes: Optional[NodesT] = None,
-                   df: Optional[pandas.DataFrame] = None,
-                   tree: Optional[HierarchyTree] = None,
-                   root: str = 'root',
-                   exogenous: Optional[List[str]] = None):
+    def __init_hts(
+        self,
+        nodes: Optional[NodesT] = None,
+        df: Optional[pandas.DataFrame] = None,
+        tree: Optional[HierarchyTree] = None,
+        root: str = "root",
+        exogenous: Optional[List[str]] = None,
+    ):
 
         if not nodes and not df:
             if not tree:
-                raise InvalidArgumentException('Either nodes and df must be passed, or a pre-built hierarchy tree')
+                raise InvalidArgumentException(
+                    "Either nodes and df must be passed, or a pre-built hierarchy tree"
+                )
             else:
                 self.nodes = tree
         else:
-            self.nodes = HierarchyTree.from_nodes(nodes=nodes, df=df, exogenous=exogenous, root=root)
+            self.nodes = HierarchyTree.from_nodes(
+                nodes=nodes, df=df, exogenous=exogenous, root=root
+            )
         self.exogenous = exogenous
-        self.sum_mat = to_sum_mat(self.nodes)
+        self.sum_mat, sum_mat_labels = to_sum_mat(self.nodes)
         self._set_model_instance()
         self._init_revision()
 
     def _init_revision(self):
-        self.revision_method = RevisionMethod(sum_mat=self.sum_mat, transformer=self.transform, name=self.method)
+        self.revision_method = RevisionMethod(
+            sum_mat=self.sum_mat, transformer=self.transform, name=self.method
+        )
 
     def _set_model_instance(self):
         try:
             self.model_instance = hts_models.MODEL_MAPPING[self.model]
         except KeyError:
-            raise InvalidArgumentException(f'Model {self.model} not valid. Pick one of: {" ".join(Model.names())}')
+            raise InvalidArgumentException(
+                f'Model {self.model} not valid. Pick one of: {" ".join(ModelT.names())}'
+            )
 
-    def fit(self,
-            df: Optional[pandas.DataFrame] = None,
-            nodes: Optional[NodesT] = None,
-            tree: Optional[HierarchyTree] = None,
-            exogenous: Optional[ExogT] = None,
-            root: str = 'total',
-            distributor: Optional[DistributorBaseClass] = None,
-            disable_progressbar=defaults.DISABLE_PROGRESSBAR,
-            show_warnings=defaults.SHOW_WARNINGS,
-            **fit_kwargs: Any) -> 'HTSRegressor':
+    def fit(
+        self,
+        df: Optional[pandas.DataFrame] = None,
+        nodes: Optional[NodesT] = None,
+        tree: Optional[HierarchyTree] = None,
+        exogenous: Optional[ExogT] = None,
+        root: str = "total",
+        distributor: Optional[DistributorBaseClass] = None,
+        disable_progressbar=defaults.DISABLE_PROGRESSBAR,
+        show_warnings=defaults.SHOW_WARNINGS,
+        **fit_kwargs: Any,
+    ) -> "HTSRegressor":
 
         """
         Fit hierarchical model to dataframe containing hierarchical data as specified in the ``nodes`` parameter
@@ -197,20 +204,22 @@ class HTSRegressor(BaseEstimator, RegressorMixin):
 
         nodes = make_iterable(self.nodes, prop=None)
 
-        fit_function_kwargs = {'fit_kwargs': fit_kwargs,
-                               'low_memory': self.low_memory,
-                               'tmp_dir': self.tmp_dir,
-                               'model_instance': self.model_instance,
-                               'model_args': self.model_args,
-                               'transform': self.transform
-                               }
+        fit_function_kwargs = {
+            "fit_kwargs": fit_kwargs,
+            "low_memory": self.low_memory,
+            "tmp_dir": self.tmp_dir,
+            "model_instance": self.model_instance,
+            "model_args": self.model_args,
+            "transform": self.transform,
+        }
 
         fitted_models = _do_fit(
             nodes=nodes,
             function_kwargs=fit_function_kwargs,
             n_jobs=self.n_jobs,
-            disable_progressbar=disable_progressbar, show_warnings=show_warnings,
-            distributor=distributor
+            disable_progressbar=disable_progressbar,
+            show_warnings=show_warnings,
+            distributor=distributor,
         )
 
         for model in fitted_models:
@@ -222,11 +231,15 @@ class HTSRegressor(BaseEstimator, RegressorMixin):
 
     def __init_predict_step(self, exogenous_df: pandas.DataFrame, steps_ahead: int):
         if self.exogenous and not exogenous_df:
-            raise MissingRegressorException(f'Exogenous variables were provided at fit step, hence are required at '
-                                            f'predict step. Please pass the \'exogenous_df\' variable to predict '
-                                            f'function')
+            raise MissingRegressorException(
+                "Exogenous variables were provided at fit step, hence are required at "
+                "predict step. Please pass the 'exogenous_df' variable to predict "
+                "function"
+            )
         if not exogenous_df and not steps_ahead:
-            logger.info(f'No arguments passed for \'steps_ahead\', defaulting to predicting 1-step-ahead')
+            logger.info(
+                "No arguments passed for 'steps_ahead', defaulting to predicting 1-step-ahead"
+            )
             steps_ahead = 1
         elif exogenous_df:
             steps_ahead = len(exogenous_df)
@@ -235,17 +248,21 @@ class HTSRegressor(BaseEstimator, RegressorMixin):
                 try:
                     node.item = exogenous_df[exog_cols]
                 except KeyError:
-                    raise MissingRegressorException(f'Node {node.key} has as exogenous variables {node.exogenous} but '
-                                                    f'these columns were not found in \'exogenous_df\'')
+                    raise MissingRegressorException(
+                        f"Node {node.key} has as exogenous variables {node.exogenous} but "
+                        f"these columns were not found in 'exogenous_df'"
+                    )
         return steps_ahead
 
-    def predict(self,
-                exogenous_df: pandas.DataFrame = None,
-                steps_ahead: int = None,
-                distributor: Optional[DistributorBaseClass] = None,
-                disable_progressbar=defaults.DISABLE_PROGRESSBAR,
-                show_warnings=defaults.SHOW_WARNINGS,
-                **predict_kwargs) -> pandas.DataFrame:
+    def predict(
+        self,
+        exogenous_df: pandas.DataFrame = None,
+        steps_ahead: int = None,
+        distributor: Optional[DistributorBaseClass] = None,
+        disable_progressbar: bool = defaults.DISABLE_PROGRESSBAR,
+        show_warnings: bool = defaults.SHOW_WARNINGS,
+        **predict_kwargs,
+    ) -> pandas.DataFrame:
         """
 
         Parameters
@@ -270,12 +287,13 @@ class HTSRegressor(BaseEstimator, RegressorMixin):
         """
 
         steps_ahead = self.__init_predict_step(exogenous_df, steps_ahead)
-        predict_function_kwargs = {'fit_kwargs': predict_kwargs,
-                                   'steps_ahead': steps_ahead,
-                                   'low_memory': self.low_memory,
-                                   'tmp_dir': self.tmp_dir,
-                                   'predict_kwargs': predict_kwargs,
-                                   }
+        predict_function_kwargs = {
+            "fit_kwargs": predict_kwargs,
+            "steps_ahead": steps_ahead,
+            "low_memory": self.low_memory,
+            "tmp_dir": self.tmp_dir,
+            "predict_kwargs": predict_kwargs,
+        }
 
         fit_models = _model_mapping_to_iterable(self.hts_result.models, self.nodes)
         results = _do_predict(
@@ -284,8 +302,7 @@ class HTSRegressor(BaseEstimator, RegressorMixin):
             n_jobs=self.n_jobs,
             disable_progressbar=disable_progressbar,
             show_warnings=show_warnings,
-            distributor=distributor
-
+            distributor=distributor,
         )
         for key, forecast, error, residual in results:
             self.hts_result.forecasts = (key, forecast)
@@ -293,29 +310,24 @@ class HTSRegressor(BaseEstimator, RegressorMixin):
             self.hts_result.residuals = (key, residual)
         return self._revise(steps_ahead=steps_ahead)
 
-    def _revise(self, steps_ahead=1):
-        logger.info(f'Reconciling forecasts using {self.revision_method}')
+    def _revise(self, steps_ahead: int = 1) -> pandas.DataFrame:
+        logger.info(f"Reconciling forecasts using {self.revision_method}")
         revised = self.revision_method.revise(
             forecasts=self.hts_result.forecasts,
             mse=self.hts_result.errors,
-            nodes=self.nodes
+            nodes=self.nodes,
         )
 
         revised_columns = list(make_iterable(self.nodes))
         revised_index = self._get_predict_index(steps_ahead=steps_ahead)
-        return pandas.DataFrame(revised,
-                                index=revised_index,
-                                columns=revised_columns)
+        return pandas.DataFrame(revised, index=revised_index, columns=revised_columns)
 
-    def _get_predict_index(self, steps_ahead=1):
+    def _get_predict_index(self, steps_ahead=1) -> Any:
 
-        freq = getattr(self.nodes.item.index, 'freq', 1)
+        freq = getattr(self.nodes.item.index, "freq", 1)
         start = self.nodes.item.index[-1] + freq
         end = self.nodes.item.index[-1] + (steps_ahead * freq)
 
-        future = pandas.date_range(freq=freq,
-                                   start=start,
-                                   end=end
-                                   )
+        future = pandas.date_range(freq=freq, start=start, end=end)
 
         return self.nodes.item.index.append(future)
